#include <stdio.h>
#include <allegro.h>
#include "dialog.h"

dialog::dialog () {
 left = 0;
 top = 0;
 width = 0;
 height = 0;
 active = 0;
 buttons_count = 0;
 labels_count = 0;
}

void dialog::set_dialog (int new_left, int new_top, int new_width, int new_height, char * new_caption) {
 left = new_left;
 top = new_top;
 width = new_width;
 height = new_height;
 int i = 0;
 while (new_caption[i] != '\0' && i < MAX_LENGTH - 1) {
  caption[i] = new_caption[i];
  i++;
 }
 caption[i] = '\0';
 dialog_bmp = create_bitmap (width, height);
 update ();
}

void dialog::add_button (int new_id, int new_left, int new_top, int new_width, int new_height, char * new_caption) {
 if (buttons_count < MAX_BUTTONS) {
  buttons[buttons_count].set_button (new_id, new_left, new_top, new_width, new_height, new_caption);
  buttons_count++;
  update ();
 }
}

void dialog::add_label (int new_id, int new_left, int new_top, int new_width, int new_height, char * new_caption) {
 if (labels_count < MAX_LABELS) {
  labels[labels_count].set_label (new_id, new_left, new_top, new_width, new_height, new_caption);
  labels_count++;
  update ();
 }
}

void dialog::button_set_caption (int id, char * new_caption) {
 for (int i = 0; i < buttons_count; i++) {
  if (buttons[i].get_id () == id)
   buttons[i].set_caption (new_caption);
 }
}

void dialog::label_set_caption (int id, char * new_caption) {
 for (int i = 0; i < labels_count; i++) {
  if (labels[i].get_id () == id)
   labels[i].set_caption (new_caption);
 }
}

int dialog::process_keys () {
 int biggest_id = 0;
 int active_id = 0;
 int smallest_id = MAX_BUTTONS * 100;
 for (int i = 0; i < buttons_count; i++) {
  if (buttons[i].get_id () > biggest_id)
   biggest_id = buttons[i].get_id ();
  if (buttons[i].get_active ())
   active_id = buttons[i].get_id ();
  if (buttons[i].get_id () < smallest_id)
   smallest_id = buttons[i].get_id ();
 }

 if (key[KEY_DOWN] || key[KEY_TAB]) {
  active_id++;
  if (active_id > biggest_id) {
   active_id = smallest_id;
  }
  while (key[KEY_DOWN] || key[KEY_TAB]) {}
 }

 if (key[KEY_UP]) {
  active_id--;
  if (active_id <= 0) {
   active_id = biggest_id;
  }
  while (key[KEY_UP]) {}
 }

 if (biggest_id != 0) {
  if (active_id == 0)
   active_id = smallest_id;
  for (int i = 0; i < buttons_count; i++) {
   if (buttons[i].get_id () == active_id) {
    buttons[i].set_active ();
   } else {
    buttons[i].set_inactive ();
   }
  }
  update ();
 }

 if (key[KEY_ENTER]) {
  while (key[KEY_ENTER]) {}
  return active_id;
 } else if (key[KEY_ESC]) {
  while (key[KEY_ESC]) {}
  return -1;
 } else {
  return 0;
 }
}

void dialog::update () {
 rectfill (dialog_bmp, 0, 0, width - 1, height - 1, back_color);
 rect (dialog_bmp, 0, 0, width - 1, height - 1, border_color);
 rectfill (dialog_bmp, 0, 0, width - 1, 20, caption_color);
 rect (dialog_bmp, 0, 0, width - 1, 20, border_color);
 textout (dialog_bmp, tahoma, caption, 4, 3, font_color);
 for (int i = 0; i < buttons_count; i++) {
  buttons[i].draw (dialog_bmp);
 }
 for (int i = 0; i < labels_count; i++) {
  labels[i].draw (dialog_bmp);
 }
}

void dialog::draw (BITMAP * output) {
 blit (dialog_bmp, output, 0, 0, left, top, width, height);
}

button::button () {
 id = 0;
 left = 0;
 top = 0;
 width = 0;
 height = 0;
 active = false;
}

void button::set_button (int new_id, int new_left, int new_top, int new_width, int new_height, char * new_caption) {
 id = new_id;
 left = new_left;
 top = new_top;
 width = new_width;
 height = new_height;
 int i = 0;
 while (*new_caption != '\0' && i < MAX_LENGTH - 1) {
  caption[i] = *new_caption;
  new_caption++;
  i++;
 }
 caption[i] = '\0';
 button_bmp = create_bitmap (width, height);
 set_inactive ();
}

void button::set_caption (char * new_caption) {
 int i = 0;
 while (*new_caption != '\0' && i < MAX_LENGTH - 1) {
  caption[i] = *new_caption;
  new_caption++;
  i++;
 }
 caption[i] = '\0';
}

void button::set_active () {
 active = true;
 rectfill (button_bmp, 0, 0, width - 1, height - 1, active_color);
 rect (button_bmp, 0, 0, width - 1, height - 1, border_color);
 textout_centre (button_bmp, tahoma, caption, width / 2, height / 2 - 7, font_color);
}
void button::set_inactive () {
 active = false;
 rectfill (button_bmp, 0, 0, width - 1, height - 1, caption_color);
 rect (button_bmp, 0, 0, width - 1, height - 1, border_color);
 textout_centre (button_bmp, tahoma, caption, width / 2, height / 2 - 7, font_color);
}

bool button::get_active () {
 return active;
}

int button::get_id () {
 return id;
}

void button::draw (BITMAP * output) {
 blit (button_bmp, output, 0, 0, left, top, width, height);
}

label::label () {
 id = 0;
 left = 0;
 top = 0;
 width = 0;
 height = 0;
}

void label::set_label (int new_id, int new_left, int new_top, int new_width, int new_height, char * new_caption) {
 id = new_id;
 left = new_left;
 top = new_top;
 width = new_width;
 height = new_height;
 int i = 0;
 while (*new_caption != '\0' && i < MAX_LENGTH - 1) {
  caption[i] = *new_caption;
  new_caption++;
  i++;
 }
 caption[i] = '\0';
 label_bmp = create_bitmap (width, height);
}

void label::set_caption (char * new_caption) {
 int i = 0;
 while (*new_caption != '\0' && i < MAX_LENGTH - 1) {
  caption[i] = *new_caption;
  new_caption++;
  i++;
 }
 caption[i] = '\0';
}

int label::get_id () {
 return id;
}

void label::draw (BITMAP * output) {
 textout (output, tahoma, caption, left, top, font_color);
}
