#include <stdio.h>
#include <allegro.h>
#include "init.h"
#include "game.h"
#include "dialog.h"

bool game_running;
bool game_over;
bool getready;
BITMAP * sideframe = NULL;
BITMAP * meter = NULL;
BITMAP * health = NULL;
BITMAP * multiplier = NULL;
MIDI * midi_music;
ship player;
int level;
int score;
float back_delta;

void game_pre_init () {
 init_molecule ();
 init_ship ();
 init_jellys ();
 meter = load_pcx ("data/meter.pcx", pal);
 health = load_pcx ("data/health.pcx", pal);
 multiplier = load_pcx ("data/multiplier.pcx", pal);
 sideframe = create_bitmap (160, 480);

 rectfill (sideframe, 0, 0, 160, 480, makecol (12, 59, 164));
 blit (title, sideframe, 0, 0, 20, 10, 120, 40);
 blit (health, sideframe, 0, 0, 24, 144, 32, 32);
 blit (multiplier, sideframe, 0, 0, 104, 144, 32, 32);
 draw_sprite (sideframe, h_ionbmp, 36, 316);
 draw_sprite (sideframe, oh_ionbmp, 104, 304);
 destroy_bitmap (title);
 destroy_bitmap (health);
 destroy_bitmap (multiplier);

 level = 1;
 score = 0;
 game_over = false;
}

void game_init () {
 if (sound_installed && (midi_music = load_midi ("data/mars.mid")) && !disable_sound)
  play_midi(midi_music, 1);
 player.init ();
}

void game_input () {
 if (key[KEY_UP])
  player.accelerate_up ();
 if (key[KEY_DOWN])
  player.accelerate_down ();
 if (key[KEY_RIGHT])
  player.accelerate_right ();
 if (key[KEY_LEFT])
  player.accelerate_left ();
 if (key[KEY_ESC] || key[KEY_Q])
  game_running = false;
 if (key[KEY_F12])
  make_screen_shot ();
 // if (key[KEY_C])
 //  player.cheat ();
 if (key[KEY_P]) {
  game_pause ();
 }
}

void game_pause () {
 int old_fps = fpscounter;
 while (key[KEY_P]) {}
 textout_centre (screen, tahoma, "Game paused, press P to continue...", 240, 240, makecol (255, 255, 255));
 while (!key[KEY_P]) {}
 while (key[KEY_P]) {}
 counter = 0;
 while (fpscounter != 0) {}
 fpscounter = old_fps;
}

void game_process () {
 float delta = 1.0 / (fpscounter + 1); // calculate delta for indepent game speed
 player.set_delta (delta); // set the delta value (for indepent game speed)
 player.game_cycle (); // move the ship and do other things each game loop
 set_delta_molecules (delta); // set the delta value (for indepent game speed)
 game_cycle_molecules (); // move the molecules
 set_delta_jellys (delta);
 set_speed_jellys (40.0);
 game_cycle_jellys ();
 test_collisions (); // test if ship collides with one or more ions
 if (player.get_h2o_molecules () >= level * 10) {
  level++;
  add_jelly (player.get_pos_x (), player.get_pos_y ());
 }
 score = 0;
 for (int i = 0; i < level - 1; i++) {
  score += (i + 1) * 10;
 }
 score += (player.get_h2o_molecules () % 10) * level;
 // create molecules if molecules are destroyed 
 while (molecules_count < 2 + level * 2) {
  add_molecule (level);
 }
 // check for end game condition
 if (player.get_health () <= 0) {
  game_running = false;
  game_over = true;
 }
 // scrolling background
 back_delta += delta * 10 * level;
 if (back_delta >= 100)
  back_delta -= 100;
}

void game_draw () {
 // clear_bitmap (buffer); // instead of clear, draw background...
 for (int i = 0; i < 6; i++) {
  for (int j = 0; j < 5; j++) {
   blit (back, buffer, 0, 0, j * 100, i * 100 - 100 + int (back_delta), 100, 100);
  }
 }
 line (buffer, 480, 0, 480, 480, makecol (255, 255, 255));
 // textprintf (buffer, font, 0, 0, makecol (255, 255, 255), "FPS: %d", fpscounter);
 draw_molecules (buffer);
 draw_jellys (buffer);
 player.draw (buffer);
 blit (sideframe, buffer, 0, 0, 480, 0, 160, 480);
 draw_meter (buffer, meter, 500, 180, makecol (255, 0, 0), player.get_health ());
 draw_meter (buffer, meter, 580, 180, makecol (255, 255, 255), (player.get_h2o_molecules () % 10) * 10);
 draw_meter (buffer, meter, 500, 340, makecol (0, 0, 255), player.get_h_ions () * 20);
 draw_meter (buffer, meter, 580, 340, makecol (0, 0, 255), player.get_oh_ions () * 20);

 textprintf (buffer, tahoma, 502, 62, makecol (0, 0, 0), "Score: %d", score);
 textprintf (buffer, tahoma, 500, 60, makecol (255, 255, 255), "Score: %d", score);
 textprintf (buffer, tahoma, 502, 82, makecol (0, 0, 0), "H20 molecules: %d", player.get_h2o_molecules ());
 textprintf (buffer, tahoma, 500, 80, makecol (255, 255, 255), "H20 molecules: %d", player.get_h2o_molecules ());
 textprintf (buffer, tahoma, 502, 102, makecol (0, 0, 0), "Level: %d", level);
 textprintf (buffer, tahoma, 500, 100, makecol (255, 255, 255), "Level: %d", level);
 
 if (getready)
  textout_centre (buffer, tahoma, "Get ready for start...", 240, 240, makecol (255, 255, 255));
 blit (buffer, screen, 0, 0, 0, 0, 640, 480);
}

void game_postgame () {
 while (key[KEY_ESC] || key[KEY_Q]) {}
 char your_score[MAX_LENGTH] = "Your score is: ";
 append_integer (your_score, score);
 dialog game_over_menu;
 game_over_menu.set_dialog (220, 140, 200, 200, "Game over!");
 game_over_menu.add_label (1, 10, 50, 100, 8, your_score);
 game_over_menu.add_button (2, 60, 120, 80, 25, "OK");
 game_over_menu.draw (screen);
 while (!(game_over_menu.process_keys () == -1 || game_over_menu.process_keys () == 2)) {}
}

void game_terminate () {
 stop_midi ();
 destroy_molecules ();
 destroy_jellys ();
}

void game_loop () {
 game_pre_init ();
 // testing game speed
 game_running = true;
 int old_fps = fpscounter;
 getready = true;
 counter = 1;
 player.init (); // reset the ship
 while (old_fps == fpscounter) {
  game_input ();
  game_draw ();
  counter++;
 }
 old_fps = fpscounter;
 while (old_fps == fpscounter && counter != 0) {
  game_input ();
  game_draw ();
  counter++;
 }
 getready = false;
 game_init ();
 while (game_running && !game_over) {
  game_input ();
  game_process ();
  game_draw ();
  counter++;
 }
 game_postgame ();
 game_terminate ();
}

void append_integer (char * string, int integer) {
 char temp[MAX_LENGTH];
 while (*string != '\0') {
  *string++;
 }
 int i = 0;
 if (integer == 0) {
  temp[0] = '0';
  i++;
 }
 while (integer > 0) {
  temp[i] = (integer % 10) + 48;
  integer = integer / 10;
  i++;
 }
 temp[i] = '\0';
 i--;
 while (i >= 0) {
  *string = temp[i];
  *string++;
  i--;
 }
 *string = '\0';
}

void draw_meter (BITMAP * output, BITMAP * input, int left, int top, int color, int percentile) {
 if (percentile > 99) {
  percentile = 99;
 }
 int width = input->w, height = input->h;
 rectfill (output, left, top, left + width - 1, top + (height * (100 - percentile)) / 100 - 1, makecol (0, 0, 0));
 rectfill (output, left, top + (height * (100 - percentile)) / 100 - 1, left + width - 1, top + height - 1,  color);
 draw_sprite (output, meter, left, top);
}

int test_collisions () {
 int collisions = 0;
 for (int i = 0; i < molecules_count; i++) {
  if (player.test_collision (molecules[i])) {
   switch (molecules[i].get_ion ()) {
    case h:
     player.collected_h_ion ();
     break;
    case oh:
     player.collected_oh_ion ();
     break;
   }
   for (int j = i; j < molecules_count - 1; j++) {
    molecules[j] = molecules[j + 1];
   }
   molecules_count--;
   collisions++;
  }
 }
 return collisions;
}

void make_screen_shot () {
 textprintf (screen, font, 0, 8, makecol (255, 255, 255), "Making a screenshot of the buffer....");
 save_tga ("screenshot.tga", buffer, pal);
}
