#include <stdio.h>
#include <math.h>
#include <time.h>
#include <allegro.h>
#include "init.h"
#include "molecule.h"

BITMAP * h_ionbmp;
BITMAP * oh_ionbmp;

molecule * molecules;
int molecules_count;
int molecules_max;
int molecules_step;

void init_molecule () {
 // set_color_conversion (COLORCONV_KEEP_TRANS);
 h_ionbmp = load_pcx ("data/h-ion.pcx", pal);
 oh_ionbmp = load_pcx ("data/oh-ion.pcx", pal);
 srand (time (NULL));

 molecules_count = 0;
 molecules_max = 0;
 molecules_step = 4;
 molecules = (molecule *) malloc (molecules_max * sizeof (molecule));
}

void add_molecule (int level) {
 molecules_count++;
 if (molecules_count > molecules_max) {
  molecules_max += molecules_step;
  molecules = (molecule *) realloc (molecules, molecules_max * sizeof (molecule));
 }
 ion_types new_ion;
 if (int (random (2)) == 1)
  new_ion = h;
 else
  new_ion = oh;
 molecules[molecules_count - 1].set_molecule (new_ion, level);
}

void set_delta_molecules (float new_delta) {
 for (int i = 0; i < molecules_count; i++)
  molecules[i].set_delta (new_delta);
}

void game_cycle_molecules () {
 for (int i = 0; i < molecules_count; i++)
  molecules[i].game_cycle ();
 for (int i = 0; i < molecules_count; i++) {
  if (!molecules[i].get_visible ()) {
   for (int j = i; j < molecules_count - 1; j++)
    molecules[j] = molecules[j + 1];
   molecules_count--;
  }
 }
}

void draw_molecules (BITMAP *output) {
 for (int i = 0; i < molecules_count; i++)
  molecules[i].draw (output);
}

void destroy_molecules () {
 free (molecules);
}

float random (float max_random) {
 return (rand () * max_random) / RAND_MAX;
}

molecule::molecule () {
 delta = 0.0;
}

void molecule::set_molecule (ion_types new_ion, int level) {
 int random_pattern;
 ion = new_ion;
 visible = true;
 switch (ion) {
  case h:
   size = 8;
   break;
  case oh:
   size = 32;
   break;
 }
 direction = 0;
 pos_x = int (random (400)) + 40;
 pos_y = 480 + size;
 if (level > 4)
  random_pattern = 4;
 else
  random_pattern = level;
 pattern = (pattern_types) int (random (random_pattern));
 if (pattern == lineair)
  pattern_cycle = random (1.4) - 0.7;
 speed = random (10 + level * 10) + 40;
}

void molecule::set_delta (float new_delta) {
 delta = new_delta;
}

void molecule::game_cycle () {
 switch (pattern) {
  case straight:
   pos_y -= speed * delta;
   break;
  case lineair:
   pos_y -= cos (pattern_cycle) * speed * delta;
   pos_x += sin (pattern_cycle) * speed * delta;
   break;
  case circles:
   pos_y -= sin (pattern_cycle) * speed * delta + speed * delta;
   pos_x += cos (pattern_cycle) * speed * delta * 4;
   pattern_cycle += PI * delta;
   break;
  case wave:
   pos_y -= speed * delta;
   pos_x += cos (pattern_cycle) * speed * delta * 4;
   pattern_cycle += PI * delta;
   break;
 }
 if (pos_x + size < 0 || pos_x - size > 480 || pos_y + size < 0 || pos_y - size > 480)
  visible = false;
 direction += (PI) * delta;
}

void molecule::draw (BITMAP * output) {
 if (visible) {
  switch (ion) {
   case h:
    draw_sprite (output, h_ionbmp, int (pos_x) - 4, 480 - int (pos_y) - 4);
    break;
   case oh:
    draw_sprite (output, oh_ionbmp, int (pos_x) - 16, 480 - int (pos_y) - 16);
    break;
  }
  // circle (output, int (pos_x), 480 - int (pos_y), size / 2, makecol (0, 0, 255));
 }
}

int molecule::get_pos_x () {
 return pos_x;
}

int molecule::get_pos_y () {
 return pos_y;
}

int molecule::get_size () {
 return size;
}

ion_types molecule::get_ion () {
 return ion;
}

bool molecule::get_visible () {
 return visible;
}
