#include <stdio.h>
//#include <math.h>
#include <allegro.h>
#include "init.h"
#include "ship.h"

BITMAP * nano_ship;

void init_ship () {
 nano_ship = load_pcx ("data/ship.pcx", pal);
}

ship::ship () {
 init ();
}

void ship::init () {
 pos_x = 240;
 pos_y = 40;
 speed_x = 0;
 speed_y = 0;
 size = 80;
 h_ions = 0;
 oh_ions = 0;
 h2o_molecules = 0;
 health = 100;
 delta = 0.0;
}

void ship::accelerate_up () {
 speed_y += delta * 128;
 if (speed_y > 128.0)
  speed_y = 128.0;
}

void ship::accelerate_down () {
 speed_y -= delta * 128;
 if (speed_y < -128.0)
  speed_y = -128.0;
}

void ship::accelerate_left () {
 speed_x -= delta * 128;
 if (speed_x < -128.0)
  speed_x = -128.0;
}

void ship::accelerate_right () {
 speed_x += delta * 128;
 if (speed_x > 128.0)
  speed_x = 128.0;
}

void ship::collected_h_ion () {
 if (oh_ions > 0) {
  oh_ions--;
  h2o_molecules++;
 } else {
  h_ions++;
 }
}

void ship::collected_oh_ion () {
 if (h_ions > 0) {
  h_ions--;
  h2o_molecules++;
 } else {
  oh_ions++;
 }
}

void ship::set_delta (float new_delta) {
 delta = new_delta;
}

void ship::game_cycle () {
 if (abs (h_ions - oh_ions) > 4)
  health -= delta * abs (h_ions - oh_ions) * 0.1;
 pos_x += speed_x * delta;
 pos_y += speed_y * delta;
 int half_size = size / 2;
 if (pos_x - half_size < 0) {
  pos_x = half_size;
  speed_x = 0;
 }
 if (pos_x + half_size > 480) {
  pos_x = 480 - half_size;
  speed_x = 0;
 }
 if (pos_y - half_size < 0) {
  pos_y = half_size;
  speed_y = 0;
 }
 if (pos_y + half_size > 480) {
  pos_y = 480 - half_size;
  speed_y = 0;
 }
}

bool ship::test_collision (molecule test_molecule) {
 bool collision = false;
 int m_pos_x = test_molecule.get_pos_x ();
 int m_pos_y = test_molecule.get_pos_y ();
 int m_size = test_molecule.get_size ();
 if ((m_pos_x - pos_x) * (m_pos_x - pos_x) + (m_pos_y - pos_y) * (m_pos_y - pos_y) < (m_size + size) * (m_size + size) * 0.25)
  collision = true;
 return collision;
}

void ship::draw (BITMAP *output) {
 draw_sprite (output, nano_ship, int (pos_x) - 40, 440 - int (pos_y));
 // circle (output, pos_x, 480 - int (pos_y), size / 2, makecol (255, 0, 0));
}

int ship::get_health () {
 return int (health);
}

int ship::get_h_ions () {
 return h_ions;
}

int ship::get_oh_ions () {
 return oh_ions;
}

int ship::get_h2o_molecules () {
 return h2o_molecules;
}

int ship::get_pos_x () {
 return int (pos_x);
}

int ship::get_pos_y () {
 return int (pos_y);
}

void ship::cheat () {
 h2o_molecules += 10;
}
